#!/usr/bin/env Rscript

library(multcomp)
library(lme4)

clones <- read.table("../data/clones.txt", header=TRUE)

clones$logSize <- log(clones$Size)
clones$GenotypeC2 <- factor(ifelse(clones$GenotypeC=="WT", "WT", "Expr"))
clones$Manip <- interaction(clones$GenotypeL, clones$Sugar, clones$GenotypeC2)
clones$Manip <- droplevels(clones$Manip)
clones$Larva <- factor(paste0(clones$Larva, "-", clones$Sugar))
clones$Series <- factor(paste0(clones$Series, "-", clones$Sugar))


contrastline <- function(gen1, gen2="T3", sug1="0%", sug2=sug1, lev=levels(clones$Manip)) {
    stopifnot(gen1 %in% levels(clones$GenotypeL), gen2 %in% levels(clones$GenotypeL))
    ll <- rep(0, length(lev))
    names(ll) <- lev
    ll[paste(gen1, sug1, "Expr", sep=".")] <- ll[paste(gen1, sug1, "Expr", sep=".")] + 1
    ll[paste(gen1, sug1, "WT", sep=".")] <- ll[paste(gen1, sug1, "WT", sep=".")] - 1
    ll[paste(gen2, sug2, "Expr", sep=".")] <- ll[paste(gen2, sug2, "Expr", sep=".")] - 1    
    ll[paste(gen2, sug2, "WT", sep=".")] <- ll[paste(gen2, sug2, "WT", sep=".")] + 1
    return(ll)
}

cont2M <- rbind(
	'Rheb+ - Co' = contrastline("RHEB", "T3"),
	'PTEN- - Co'  = contrastline("PTEN", "T3"),
	'PTEN-;Rheb+ - Co' = contrastline("PTEN-RHEB", "T3"),
	'PTEN-;Rheb+ - Rheb+' = contrastline("PTEN-RHEB", "RHEB"),
	'PTEN-;Rheb+ - PTEN-' = contrastline("PTEN-RHEB", "PTEN"))

cont5M <- rbind(
	'T3-0% - 20%' = contrastline("T3", "T3", "20%", "0%"),
	'FASN-0% - 20%'  = contrastline("FASN", "FASN", "20%", "0%"),
	'Rheb+-0% - 20%'  = contrastline("RHEB", "RHEB", "20%", "0%"),
	'FASN;Rheb+-0% - 20%' = contrastline("FASN-RHEB", "FASN-RHEB", "20%", "0%"), 
	'PTEN-0% - 20%'  = contrastline("PTEN", "PTEN", "20%", "0%"),
	'FASN-,PTEN-0% - 20%' = contrastline("FASN-PTEN", "FASN-PTEN", "20%", "0%"),
	'PTEN-;Rheb+-0% - 20%'  = contrastline("PTEN-RHEB", "PTEN-RHEB", "20%", "0%"),
	'FASN-,PTEN-;Rheb+- 0% - 20%' = contrastline("FASN-PTEN-RHEB", "FASN-PTEN-RHEB", "20%", "0%"),
	'FASN-,PTEN- - FASN-' = contrastline("FASN-PTEN", "FASN"),
	'FASN-;Rheb+ - FASN-' = contrastline("FASN-RHEB", "FASN"), 
	'FASN-;Rheb+ - Rheb+' = contrastline("FASN-RHEB", "RHEB"),
	'FASN-,PTEN- - PTEN-' = contrastline("FASN-PTEN", "PTEN"),
	'FASN-,PTEN-;Rheb+ - PTEN-;Rheb+' = contrastline("FASN-PTEN-RHEB", "PTEN-RHEB"),
	'PTEN-Ri - PTEN-' = contrastline("PTEN-Ri","PTEN"),
	'FASN-;PTEN-Ri - FASN-;PTEN-' = contrastline("FASN-PTEN-Ri","FASN-PTEN"),
	'PTEN-Ri - Co' = contrastline("PTEN-Ri","T3"),
	'FASN-;PTEN-Ri - Co' = contrastline("FASN-PTEN-Ri","T3"),
	'PTEN-Ri - FASN;PTEN-Ri'= contrastline("PTEN-Ri", "FASN-PTEN-Ri"))

cont7M <- rbind(
	'PFK1i - Co' = contrastline("PFK1-Ri", "T3"),
	'PKi - Co'   = contrastline("PK-Ri", "T3"),
	'LDHi - Co'  = contrastline("LDH-Ri", "T3"),
	'PDHi - Co'  = contrastline("PDH-Ri", "T3"),
	'PTEN-PFK1i - PTEN-'= contrastline("PTEN-PFK1-Ri", "PTEN"),
	'PTEN-PKi - PTEN-'= contrastline("PTEN-PK-Ri", "PTEN"),
	'PTEN-LDHi - PTEN-'= contrastline("PTEN-LDH-Ri", "PTEN"),
	'PTEN-PDHi - PTEN-'= contrastline("PTEN-PDH-Ri", "PTEN"),
	'Rheb+-PFK1i - Rheb+' = contrastline("RHEB-PFK1-Ri", "RHEB"),
	'Rheb+-PKi - Rheb+' = contrastline("RHEB-PK-Ri", "RHEB"),
	'Rheb+-LDHi - Rheb+' = contrastline("RHEB-LDH-Ri", "RHEB"),
	'Rheb+-PDHi - Rheb+' =contrastline("RHEB-PDH-Ri", "RHEB"))

maketable <- function(mod, cont, ...) {
	require(xtable)
	G <- glht(mod, linfct=mcp(Manip=cont))
	sm <- summary(G, test=adjusted("holm"))
	smt <- data.frame(
		'Estimate' = sm$test$coefficients,
		'Std error'= sm$test$sigma,
#~ 		'P-value' = summary(G, test=adjusted("none"))$test$pvalue,
		'Adj P-value' = sm$test$pvalue)
	rownames(smt) <- names(sm$test$coefficients)
	xtable(smt, digits=4, ...)
}

mod2 <- lmer(logSize ~ Manip+(1|Series)+(1|Larva), data=clones)

print(maketable(mod2, cont2M), type="html", file="TabS1.html")
print(maketable(mod2, cont5M), type="html", file="TabS1.html", append=TRUE)
print(maketable(mod2, cont7M, caption="Statistical tests corresponding to figures 2M, 5M, and 7M. The model tests for the difference in the log ratio of GFP vs. unlabelled clones between pairs of genotypes (Co = control), accounting for larvae and series random effects (see the Methods section). P-values were corrected for multiple testing (Holm-Bonferroni method) and quantify the risk of rejecting the true null hypothesis at the table level."), type="html", file="TabS1.html", append=TRUE)

