#!/usr/bin/env Rscript

library(xtable)

# The analysis corresponds to the data presented in Fig 6A

pupdev <- read.table("../data/development.txt", header=TRUE)

pupdev.tot <- pupdev[pupdev$Days==0,]
pupdev.rem <- pupdev[pupdev$Days!=0,]

# Part A: Emerged at Day 15

e15 <- data.frame(
	Genotype=pupdev.tot$Genotype, 
	Sugar   =pupdev.tot$Sugar,
	Replicate=pupdev.tot$Replicate,
	Total   =pupdev.tot$Emerged,
	Emerged15=sapply(pupdev.tot$Replicate, function(rr) sum(pupdev.rem$Emerged[pupdev.rem$Replicate==rr])))
	
library(lme4)
library(multcomp)


e15$GenoSug <- interaction(e15$Genotype, e15$Sugar)
m15 <- glmer(cbind(Emerged15, Total-Emerged15) ~ 0+GenoSug + (1|Replicate), family="binomial", data=e15)
s15 <- summary(glht(m15, mcp(GenoSug="Tukey")), test=adjusted("holm"))
tt <- data.frame(Effect=coef(s15), 'Std.Err'=s15$test$sigma, 'Adj.Pvalue'=s15$test$pvalues)

xt <- xtable(tt, center="centering", floating=FALSE, digits=c(0, 3, 3, 5), caption="Pairwise comparison of Emergence frequencies (logit scale) at Day 15.")
print(xt, file="TabS6.html", hline.after=c(-1,0,4), type="html")

# Part B: Non-linear model fitting
library(nlme)

#       Cumulative frequencies / probably not the most elegant way
cf <- tapply(pupdev.rem$Emerged, pupdev.rem$Replicate, cumsum)
cf <- sapply(names(cf), function(r) cf[[r]] <- cf[[r]]/pupdev.tot$Emerged[pupdev.tot$Replicate==r], simplify=FALSE)
pupdev.rem$CumFreq <- NA
for (r in levels(pupdev.rem$Replicate)) 
	pupdev.rem$CumFreq[pupdev.rem$Replicate == r] <- cf[[r]]

pupdev.rem$GenoSug <- interaction(pupdev.rem$Genotype, pupdev.rem$Sugar)
nl <- length(levels(pupdev.rem$GenoSug))
contr <- rbind('Co.0%' = c(-1, -1, 0), 'FASN.0%' = c(0, 1, -1), 'Co.10%' = c(1, 0, 0), 'FASN.10%' = c(0, 0, 1))
m0 <- matrix(0, nrow=nrow(contr), ncol=ncol(contr))
	
stv <- c(Asym=c(0.43,0.36,0.70,0.21), xmid=rep(8,nl), scal=rep(0.6, nl))
nn <- nlme(
	CumFreq ~ SSlogis(Days, Asym, xmid, scal), 
	fixed = list(Asym ~ 0+GenoSug, xmid ~ 0+GenoSug, scal ~ 0+GenoSug), 
	random = Asym + xmid ~ 1|Replicate, 
	data=pupdev.rem, start=stv, 
	control=nlmeControl(msMaxIter=500, maxIter=500))

contr <- rbind(
	'Asym: Co.10% - Co.0%'   =c(-1,0,1,0,  0,0,0,0,  0,0,0,0), 
	'Asym: FASN.0% - Co.0%'  =c(-1,1,0,0,  0,0,0,0,  0,0,0,0), 
	'Asym: FASN.10% - FASN.0%'=c(0,-1,0,1, 0,0,0,0,  0,0,0,0), 
	'xmid: Co.10% - Co.0%'   =c(0,0,0,0,  -1,0,1,0,  0,0,0,0), 
	'xmid: FASN.0% - Co.0%'  =c(0,0,0,0,  -1,1,0,0,  0,0,0,0), 
	'xmid: FASN.10% - FASN.0%'=c(0,0,0,0,  0,-1,0,1, 0,0,0,0),
	'scal: Co.10% - Co.0%'   =c(0,0,0,0,   0,0,0,0, -1,0,1,0),
	'scal: FASN.0% - Co.0%'  =c(0,0,0,0,   0,0,0,0, -1,1,0,0),
	'scal: FASN.10% - FASN.0%'=c(0,0,0,0,  0,0,0,0, 0,-1,0,1))
	
library(multcomp)

ss <- summary(glht(nn, contr),test=adjusted("holm"))
rr <- do.call(cbind, ss$test)[,c("coefficients", "sigma","pvalues")]
colnames(rr) <- c("Value","Std.Err.", "Adj.Pvalue")

xt <- xtable(rr, center="centering", floating=FALSE, digits=c(0, 3, 3, 5), caption=
		"Coefficients (logistic model with 3 parameters: asymptote (Asym), day at inflection (xmid), and horizontal stretching (scal)) for the emergence frequency analysis. The model is a non-linear mixed-effect model, with Genotype and Sugar as fixed factors for all three logistic coefficents, and Replicate as a random effect (for Asym and xmid only, due to convergence issues with the full model). The last column indicate P-values corrected for multiple testing (Holm-Bonferoni correction)." )
print(xt, file="TabS6.html", hline.after=c(-1,0,3,6,9), type="html",append=TRUE)

	
