

check.comparison <- function(comp, data=NULL) {
    if (!is.list(comp) ||  length(comp) != 2 || any(names(comp[[1]]) != names(comp[[2]])) ) {
        warning("The comparison is not a list of two lists with the same names")
        return(FALSE)
    }

    if (!is.null(data) && is.data.frame(data)) {
        if (!all(names(comp[[1]]) %in% names(data))){
            warning("All comparison names are not data frame columns")
            return(FALSE)
        }
        for (i in names(comp[[1]])) {
            for (j in 1:2) {
                lev <- comp[[j]][i]
                if (!lev %in% levels(as.factor(data[,i]))) {
                    warning("Level", lev, "is not a factor level in the data.")
                    return(FALSE)
                }
            }
        }
    }
    return(TRUE)
}

select.dat <- function(data, conditions) {
    # assuming that conditions are a proper component of a comparison
    
    Reduce('&', lapply(names(conditions), function(fact) data[,fact] == conditions[fact]))
}

sum.compname <- function(comp, letters=c(3, 3), sep1="=", sep2=",") {
    # Summarizes the name of the contrast for a nice display
    if (length(letters) == 1) letters <- rep(letters, 2)
    .minisum <- function(listfact) {
        fact <- unlist(substr(names(listfact), 0, letters[1]))
        lev  <- unlist(substr(as.character(listfact), 0, letters[2]))
        paste(fact, lev, sep=sep1, collapse=sep2)
    }
    paste(.minisum(comp[[1]]), .minisum(comp[[2]]), sep=" vs. ")
}

compareContrasts <- function(data, variable, contrasts) {
    # Compare two methods: 
    # 1. naive successive t.tests between series
    # 2. contrasts from a linear model (using the contrast library)
    
    # data: a data frame
    # contrasts: a list of comparisons.
    #  a comparison is itself a list of two lists, summarizing factor levels:
    #   list(
    #        list(factorA=levelA1, factorB=levelB1),
    #        list(factorA=levelA1, factorB=levelB2))
    
    if (length(variable) == 1) variable <- data[,variable]
    stopifnot(length(variable) == nrow(data))
    
    # first method: successive t-tests
    method1 <- do.call(rbind, lapply(contrasts, function(comp) {
        stopifnot(check.comparison(comp))
        d1 <- variable[select.dat(data, comp[[1]])]
        d2 <- variable[select.dat(data, comp[[2]])]
        tt <- t.test(d1, d2)
        data.frame( 'Estimate'  =tt$estimate[2]-tt$estimate[1],
                    'Std. error'=diff(tt$conf.int)/1.96/2,
                    't value'   =tt$statistic,
                    'Pr(>|t|)'  =tt$p.value, check.names=FALSE)
    }))
    
    # second method: linear model and contrasts
    method2 <- if (!require(contrast)) {NULL} else {
        allfactors <- unique(names(unlist(contrasts)))
        model <- as.formula(paste("variable", paste(allfactors, collapse="*"), sep='~'))
        ll <- lm(model, data=data)
        ans <- lapply(contrasts, function(comp) {
            contrast(ll, comp[[1]], comp[[2]])
        } )
        data.frame( 'Estimate'   = sapply(ans, "[[" ,"Contrast", USE.NAMES=FALSE),
                    'Std. error' = sapply(ans, "[[", "SE", USE.NAMES=FALSE),
                    't value'    = sapply(ans, "[[", "testStat", USE.NAMES=FALSE),
                    'Pr(>|t|)'   = sapply(ans, "[[", "Pvalue", USE.NAMES=FALSE), check.names=FALSE)
    }
    
    
    rownames(method1) <- rownames(method2) <- sapply(contrasts, sum.compname)
    
    return(list(raw.t=method1, contrasts=method2))
    
}
